@@ -27,7 +27,42 @@ Asynchronous database transactions via SQLAlchemy.
 import sys
 from twisted.internet import defer
 from twisted.python import failure
+
 import sqlalchemy as SA
+
+######################################################################
+# SA 0.4 support contributed by Ricky Iacovou,  based upon:
+#
+#   http://www.sqlalchemy.org/docs/04/intro.html#overview_migration
+#
+# Determine the version of SQLAlchemy used, 0.3 or 0.4, and set the
+# Boolean variable "SA04" accordingly.
+#
+# We could also use a capability-based approach, like:
+#
+# try:
+#     MetaData = SA.BoundMetaData
+# except AttributeError:
+#     MetaData = SA.MetaData
+#
+# However, late 0.3.x versions also supported some 0.4 constructs,
+# so better use an explicit 0.3.x -> 0.4.x cutoff in order to avoid
+# ambiguity.
+######################################################################
+_sv = SA.__version__.split ('.')
+try:
+    _v = int (_sv[0]) + (int(_sv[1]) / 10.0)
+except:
+    # Not strictly an Import Error, but close enough.
+    raise ImportError("Failed to determine SQLAlchemy version: %s", _sv)
+if _v >= 0.4:
+    SA04 = True
+else:
+    SA04 = False
+del _sv, _v
+# End of version check
+
+
 from asynqueue import ThreadQueue
 
 import misc
@@ -241,8 +276,7 @@ class AccessBroker(object):
     def _getQueue(self):
         """
         Returns a threaded task queue that is dedicated to my database
-        connection. Creates the queue if the first time the property is
-        accessed.
+        connection. Creates the queue the first time the property is accessed.
         """
         def newQueue():
             queue = ThreadQueue(1)
@@ -285,6 +319,7 @@ class AccessBroker(object):
 
         def createEngine():
             url, kw = self.engineParams
+            # The 'threadlocal' keyword value is unchanged from SA 0.3 to 0.4
             kw['strategy'] = 'threadlocal'
             return SA.create_engine(url, **kw)
         
@@ -326,10 +361,15 @@ class AccessBroker(object):
 
     def getSession(self):
         """
-        Get a comittable session object
+        Get a commitable session object
         """
         def gotConnection(connection):
-            d = self.q.call(SA.create_session, bind_to=connection, doNext=True)
+            if SA04:
+                d = self.q.call(
+                    SA.orm.create_session, bind=connection, doNext=True)
+            else:
+                d = self.q.call(
+                    SA.create_session, bind_to=connection, doNext=True)
             d.addCallback(gotSession)
             return d
 
@@ -358,7 +398,10 @@ class AccessBroker(object):
         """
         def _table():
             if not hasattr(self, '_meta'):
-                self._meta = SA.BoundMetaData(self._engine)
+                if SA04:
+                    self._meta = SA.MetaData(self._engine)
+                else:
+                    self._meta = SA.BoundMetaData(self._engine)
             indexes = {}
             for key in kw.keys():
                 if key.startswith('index_'):
